package sets;


import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;



public class CoarseSet<T> extends SequentialSet<T>
{
    // lock controlling access to the whole set
    private Lock lock = new ReentrantLock();

    public CoarseSet() {
        super();
    }

    // add while locking whole set
    @Override
    public boolean add(T item) {
        lock.lock();  // lock whole set
        try {
            return super.add(item);
        } finally {  // executed after return
            lock.unlock();
        }
    }

    // remove while locking whole set
    @Override
    public boolean remove(T item) {
        lock.lock();  // lock whole set
        try {
            return super.remove(item);
        } finally {  // executed after return
            lock.unlock();
        }
    }

    // has while locking whole set
    @Override
    public boolean has(T item) {
        lock.lock();  // lock whole set
        try {
            return super.has(item);
        } finally {  // executed after return
            lock.unlock();
        }
    }
}
